﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Web;
using Josso.Agent.Lib.Exception;
using NUnit.Framework;
using Rhino.Mocks;

namespace Josso.Agent.Lib.Tests
{
    [TestFixture]
    public class ZjsAgentTests
    {
        internal const string FAKE_GATEWAY = "https://fake.gateway";

        internal const string USERNAME = "user_name";

        private readonly ZjsAgent _agent;

        /// <summary>
        /// Build the josso agent
        /// </summary>
        public ZjsAgentTests()
        {
            _agent = ZjsAgent.Init(FAKE_GATEWAY, "login", "logout", "check", false);
        }

        [SetUp]
        public void Setup()
        {
            _agent.Josso = new ZjsMockedJosso();
        }

        /// <summary>
        /// Try to get the login page without a current url specify in the page call
        /// </summary>
        [Test, Category("SMALL")]
        public void TestGetJosssoLoginPageWithoutCurrentUrl()
        {
            HttpRequestBase request = GetMockedRequest();
            HttpSessionStateBase session = MockRepository.GenerateMock<HttpSessionStateBase>();
            try
            {
                _agent.GetJossoLoginPage(request, session);
                Assert.Fail("Exception {0} expected", typeof(NullCurrentUrlException).Name);
            }
            catch (NullCurrentUrlException e)
            {
                Assert.AreEqual("Specify a current url in the page call", e.Message);
            }
        }

        /// <summary>
        /// Get the josso current login url
        /// </summary>
        [Test, Category("SMALL")]
        public void TestGetJosssoLoginPage()
        {
            HttpRequestBase request = GetMockedRequest();
            request.Stub(r => r["josso_current_url"]).Return("current_url");
            HttpSessionStateBase session = MockRepository.GenerateMock<HttpSessionStateBase>();

            string jossoLoginPage = _agent.GetJossoLoginPage(request, session);
            Assert.AreEqual("https://fake.gateway/josso/signon/login.do"
                            + "?josso_back_to=http://server_name:666/check"
                            + "&josso_partnerapp_host=server_name"
                            + "&josso_partnerapp_ctx=current_url",
                            jossoLoginPage);
        }

        /// <summary>
        /// Get the check page action
        /// </summary>
        [Test, Category("SMALL")]
        public void TestCheckPageActionWithNoAssertionId()
        {
            HttpRequestBase request = GetMockedRequest();
            request.Stub(r => r["josso_partnerapp_ctx"]).Return("current_url");
            HttpSessionStateBase session = MockRepository.GenerateMock<HttpSessionStateBase>();
            HttpResponseBase response = MockRepository.GenerateMock<HttpResponseBase>();
            _agent.JossoCheckPageAction(request, session, response);

            response.AssertWasCalled(r => r.Redirect("https://fake.gateway/josso/signon/login.do"
                                                     + "?josso_back_to=http://server_name:666/check"
                                                     + "&josso_partnerapp_host=server_name"
                                                     + "&josso_partnerapp_ctx=current_url", true));
        }

        /// <summary>
        /// Get the check page action when assertion is wrong
        /// </summary>
        [Test, Category("SMALL")]
        public void TestCheckPageActionWithWrongAssertion()
        {
            HttpRequestBase request = GetMockedRequest();
            request.Stub(r => r["josso_partnerapp_ctx"]).Return("current_url");
            request.Stub(r => r["josso_assertion_id"]).Return(ZjsMockedJosso.WRONG_ASSERT);
            HttpSessionStateBase session = MockRepository.GenerateMock<HttpSessionStateBase>();
            HttpResponseBase response = MockRepository.GenerateMock<HttpResponseBase>();
            _agent.JossoCheckPageAction(request, session, response);

            response.AssertWasCalled(r => r.Redirect("https://fake.gateway/josso/signon/login.do"
                                                     + "?josso_back_to=http://server_name:666/check"
                                                     + "&josso_partnerapp_host=server_name"
                                                     + "&josso_partnerapp_ctx=current_url", true));
        }

        /// <summary>
        /// Get the check page action
        /// </summary>
        [Test, Category("SMALL")]
        public void TestCheckPageAction()
        {
            HttpRequestBase request = GetMockedRequest();
            request.Stub(r => r["josso_partnerapp_ctx"]).Return("current_url");
            request.Stub(r => r["josso_assertion_id"]).Return("correct");
            HttpSessionStateBase session = MockRepository.GenerateMock<HttpSessionStateBase>();
            HttpResponseBase response = MockRepository.GenerateMock<HttpResponseBase>();
            HttpCookieCollection cookies = new HttpCookieCollection();
            response.Stub(r => r.Cookies).Return(cookies);

            _agent.JossoCheckPageAction(request, session, response);

            response.AssertWasCalled(r => r.Redirect("current_url"));
            Assert.AreEqual(1, cookies.AllKeys.Length);
            Assert.AreEqual(ZjsBaseAgent.COOKIE_NAME, cookies.AllKeys[0]);
            HttpCookie cookie = cookies[cookies.AllKeys[0]];
            Assert.IsNotNull(cookie);
            Assert.AreEqual(null, cookie.Domain);
            Assert.AreEqual(ZjsBaseAgent.COOKIE_NAME, cookie.Name);
            Assert.AreEqual(ZjsMockedJosso.SSO_SESSION, cookie.Value);
            Assert.AreEqual(DateTime.MinValue, cookie.Expires);
        }

        /// <summary>
        /// Test to create the login url
        /// </summary>
        [Test, Category("SMALL")]
        public void TestCreateLoginUrl()
        {
            HttpRequestBase request = GetMockedRequest();
            string loginUrl = _agent.JossoCreateLoginUrl(request);
            Assert.AreEqual("http://server_name:666/login"
                            + "?josso_current_url=http://www.virtuoz.com/test",
                            loginUrl);
        }

        /// <summary>
        /// Test to create the logout url
        /// </summary>
        [Test, Category("SMALL")]
        public void TestCreateLogoutUrl()
        {
            HttpRequestBase request = GetMockedRequest();
            string logoutUrl = _agent.JossoCreateLogoutUrl(request);
            Assert.AreEqual("http://server_name:666/logout"
                            + "?josso_current_url=http://www.virtuoz.com/test",
                            logoutUrl);

            logoutUrl = _agent.JossoCreateLogoutUrl(request, "return_url");
            Assert.AreEqual("http://server_name:666/logout"
                            + "?josso_current_url=return_url",
                            logoutUrl);
        }

        /// <summary>
        /// Test the logout page action.
        /// </summary>
        [Test, Category("SMALL")]
        public void TestLogoutPageActionWithoutCurrentUrl()
        {
            HttpRequestBase request = GetMockedRequest();
            HttpResponseBase response = MockRepository.GenerateMock<HttpResponseBase>();

            try
            {
                _agent.JossoLogoutPageAction(request, response);
                Assert.Fail("Exception {0} expected", typeof(NullCurrentUrlException).Name);
            }
            catch (NullCurrentUrlException e)
            {
                Assert.AreEqual("Specify a current url in the page call", e.Message);
            }
        }

        /// <summary>
        /// Test the logout page action.
        /// </summary>
        [Test, Category("SMALL")]
        public void TestLogoutPageActionWithNoCookie()
        {
            HttpRequestBase request = GetMockedRequest();
            request.Stub(r => r["josso_current_url"]).Return("current_url");

            HttpResponseBase response = MockRepository.GenerateMock<HttpResponseBase>();

            _agent.JossoLogoutPageAction(request, response);

            response.AssertWasCalled(r => r.Redirect("https://fake.gateway/josso/signon/logout.do"
                                                     + "?josso_back_to=current_url"));
        }

        /// <summary>
        /// Test the logout page action.
        /// </summary>
        [Test, Category("SMALL")]
        public void TestLogoutPageAction()
        {
            //mock request
            HttpRequestBase request = GetMockedRequest();
            request.Stub(r => r["josso_current_url"]).Return("current_url");
            request.Cookies.Add(new HttpCookie(ZjsBaseAgent.COOKIE_NAME, ZjsMockedJosso.SSO_SESSION));
            // mock response
            HttpResponseBase response = MockRepository.GenerateMock<HttpResponseBase>();
            response.Stub(r => r.Cookies).Return(new HttpCookieCollection());

            _agent.JossoLogoutPageAction(request, response);

            Assert.AreEqual(ZjsMockedJosso.SSO_SESSION, ((ZjsMockedJosso)_agent.Josso).LogoutSession);
            response.AssertWasCalled(r => r.Redirect("https://fake.gateway/josso/signon/logout.do"
                                                     + "?josso_back_to=current_url"));
        }

        /// <summary>
        /// Test the logout page action.
        /// </summary>
        [Test, Category("SMALL")]
        public void TestLogoutPageActionWithResponseBeingRedirected()
        {
            HttpRequestBase request = GetMockedRequest();
            request.Stub(r => r["josso_current_url"]).Return("current_url");
            HttpResponseBase response = MockRepository.GenerateMock<HttpResponseBase>();
            response.Stub(r => r.IsRequestBeingRedirected).Return(true);

            _agent.JossoLogoutPageAction(request, response);
        }

        /// <summary>
        /// Test the logout page action.
        /// </summary>
        [Test, Category("SMALL")]
        public void TestTryGetJossoLoginWithoutCookie()
        {
            HttpRequestBase request = GetMockedRequest();
            HttpSessionStateBase session = MockRepository.GenerateMock<HttpSessionStateBase>();

            string login;
            string redirect;
            bool result = _agent.TryGetJossoLogin(request, session, out login, out redirect);

            Assert.IsFalse(result);
            Assert.AreEqual(null, login);
            Assert.AreEqual("https://fake.gateway/josso/signon/login.do"
                            + "?josso_back_to=http://server_name:666/check"
                            + "&josso_partnerapp_host=server_name"
                            + "&josso_partnerapp_ctx=http%3a%2f%2fwww.virtuoz.com%2ftest", redirect);
        }

        /// <summary>
        /// Test the logout page action.
        /// </summary>
        [Test, Category("SMALL")]
        public void TestTryGetJossoLogin()
        {
            HttpRequestBase request = GetMockedRequest();
            request.Cookies.Add(new HttpCookie(ZjsBaseAgent.COOKIE_NAME, ZjsMockedJosso.SSO_SESSION));
            HttpSessionStateBase session = MockRepository.GenerateMock<HttpSessionStateBase>();

            ((ZjsMockedJosso)_agent.Josso).UserName = USERNAME;
            ((ZjsMockedJosso)_agent.Josso).Properties = new Dictionary<string, string>();

            string login;
            string redirect;
            bool result = _agent.TryGetJossoLogin(request, session, out login, out redirect);

            Assert.IsTrue(result);
            Assert.AreEqual(USERNAME, login);
            Assert.AreEqual(null, redirect);
        }

        /// <summary>
        /// Get the current user
        /// </summary>
        [Test, Category("SMALL")]
        public void TestJossoCurrentUser()
        {
            HttpRequestBase request = GetMockedRequest();
            _agent.JossoCurrentUser(request);
        }

        /// <summary>
        /// Get a mocked request.
        /// </summary>
        /// <returns>A mocked request.</returns>
        internal static HttpRequestBase GetMockedRequest()
        {
            HttpRequestBase request = MockRepository.GenerateMock<HttpRequestBase>();
            NameValueCollection serverVariables = new NameValueCollection();
            serverVariables["server_name"] = "server_name";
            serverVariables["server_port"] = "666";
            request.Stub(r => r.Cookies).Return(new HttpCookieCollection());
            request.Stub(r => r.ServerVariables).Return(serverVariables);
            request.Stub(r => r.Url).Return(new Uri("http://www.virtuoz.com/test"));
            return request;
        }

        internal class ZjsMockedJosso : ZjsJosso
        {
            public const string WRONG_ASSERT = "WRONG";

            public const string SSO_SESSION = "SSO_SESSION";

            public ZjsMockedJosso() : base(FAKE_GATEWAY) { }

            public string LogoutSession { get; private set; }

            public string UserName { get; set; }

            public Dictionary<string, string> Properties { get; set; }

            public override bool ResolveAuthenticationAssertion(string assertionId, out string ssoSession)
            {
                if (assertionId == WRONG_ASSERT)
                {
                    ssoSession = null;
                    return false;
                }

                ssoSession = SSO_SESSION;
                return true;
            }

            public override void GlobalLogout(string sessionId)
            {
                LogoutSession = sessionId;
            }

            public override bool AccessSession(string sessionId)
            {
                return sessionId != WRONG_ASSERT;
            }

            public override string GetUserName(string sessionId)
            {
                return UserName;
            }

            public override Dictionary<string, string> GetProperties(string sessionId)
            {
                return Properties;
            }

            public override bool IdentifyUser(string user, string password, out string ssoSession)
            {
                ssoSession = SSO_SESSION;
                return true;
            }
        }
    }
}