﻿using System;
using System.Collections.Specialized;
using System.Configuration;
using System.Web;
using Josso.Agent.Lib.Exception;

namespace Josso.Agent.Lib
{
    /// <summary>
    /// this class represent the client side agent of josso
    /// </summary>
    public class ZjsAgent : ZjsBaseAgent, YjsAgent
    {
        /// <summary>
        /// The request param name for the BackTo URL.
        /// </summary>
        private const string JOSSO_PARTNER_APP_CTX = "josso_partnerapp_ctx";

        /// <summary>
        /// the page for login
        /// </summary>
        private string _jossoLoginPage;

        /// <summary>
        /// the page for logout
        /// </summary>
        private string _jossoLogoutPage;

        /// <summary>
        /// the page for security checking
        /// </summary>
        private string _jossoCheckPage;

        /// <summary>
        /// the login page (on the gateway)
        /// </summary>
        private string _gatewayLogin;

        /// <summary>
        /// the logout page (on the gateway)
        /// </summary>
        private string _gatewayLogout;

        /// <summary>
        /// constructor for the factory
        /// </summary>
        /// <param name="gatewayUrl">the url of the gateway</param>
        /// <param name="disableSSL">disable the SSL</param>
        private ZjsAgent(string gatewayUrl, bool disableSSL) : base(gatewayUrl, disableSSL) { }

        /// <summary>
        /// the url of the info of the user (on the josso server)
        /// </summary>
        public string JossoInfoUrl { get; private set; }

        /// <summary>
        /// the page for login
        /// </summary>
        private string jossoLoginPage
        {
            get
            {
                if (string.IsNullOrEmpty(_jossoLoginPage))
                    throw new ConfigurationErrorsException(
                        "Have you specified the JossoLoginPage configuration ?");
                return _jossoLoginPage;

            }
        }

        /// <summary>
        /// the page for logout
        /// </summary>
        private string jossoLogoutPage
        {
            get
            {
                if (string.IsNullOrEmpty(_jossoLogoutPage))
                    throw new ConfigurationErrorsException(
                        "Have you specified the JossoLogoutPage configuration ?");
                return _jossoLogoutPage;
            }
        }

        /// <summary>
        /// the page for security checking
        /// </summary>
        private string jossoCheckPage
        {
            get
            {
                if (string.IsNullOrEmpty(_jossoCheckPage))
                    throw new ConfigurationErrorsException(
                        "Have you specified the JossoCheckPage configuration ?");
                return _jossoCheckPage;
            }
        }

        /// <summary>
        /// Initialize the josso object with the configuration.
        /// This is a factory.
        /// </summary>
        /// <param name="gateway">the url of the gateway</param>
        /// <param name="jossoLoginPage">the url to the login page</param>
        /// <param name="jossoLogoutPage">the url to the logout page</param>
        /// <param name="jossoCheckPage">the url of the page for security check</param>
        /// <param name="disableSSL">disable the check on ssl</param>
        /// <returns>the agent</returns>
        public static ZjsAgent Init(string gateway, string jossoLoginPage, string jossoLogoutPage,
                                    string jossoCheckPage, bool disableSSL)
        {
            ZjsAgent agent = new ZjsAgent(gateway, disableSSL);

            // JOSSO Gateway Login/Logout urls
            agent._gatewayLogin = string.Format("{0}/josso/signon/login.do", gateway);
            agent._gatewayLogout = string.Format("{0}/josso/signon/logout.do", gateway);
            agent.JossoInfoUrl = string.Format("{0}/josso/signon/info.do", gateway);

            // JOSSO ASP Agent base code i.e. /josso-asp for http://my-host.com/josso-asp/josso.asp, etc
            agent._jossoLoginPage = jossoLoginPage;
            agent._jossoLogoutPage = jossoLogoutPage;
            agent._jossoCheckPage = jossoCheckPage;

            return agent;
        }

        /// <summary>
        /// Initialize the josso object with the configuration.
        /// This is a factory.
        /// </summary>
        /// <param name="configuration"></param>
        /// <returns>the agent</returns>
        public static ZjsAgent Init(NameValueCollection configuration)
        {
            // fecth config values
            string gateway = configuration["JossoGateway"];
            string jossologinPage = configuration["JossoLoginPage"];
            string jossologoutPage = configuration["JossoLogoutPage"];
            string jossoCheckPage = configuration["JossoCheckPage"];

            bool useSSL;
            if (string.IsNullOrEmpty(configuration["JossoUseSSL"]))
            {
                useSSL = true;
            }
            else
            {
                if (!bool.TryParse(configuration["JossoUseSSL"], out useSSL))
                {
                    useSSL = true;
                }
            }

            if (!useSSL)
            {
                Console.WriteLine("!!!! SSL is not configured !!!!");
            }

            // return the agent
            return Init(gateway, jossologinPage, jossologoutPage,
                        jossoCheckPage, !useSSL);
        }

        /// <summary>
        /// This function returns the current SSO user, if no user has been authenticated or if
        /// the SSO Session has expired, it returns null
        /// </summary>
        /// <param name="request"></param>
        /// <returns>the current user</returns>
        public ZjsUser JossoCurrentUser(HttpRequestBase request)
        {
            // Get current SSO Session ID, if any ...
            HttpCookie ssoSession = getJossoToken(request);

            if (ssoSession != null && ssoSession.Value != "")
            {
                // We have a SSO Session, find the associated user ...
                // If the session has expired, there will be no user associated to it.
                return ZjsUser.UserFactory(Josso, ssoSession.Value);
            }

            return null;
        }

        /// <summary>
        /// Returns the login URL. The client is responsible to redirect to this URL.
        /// </summary>
        /// <param name="request"></param>
        /// <param name="session"></param>
        public string GetJossoLoginPage(HttpRequestBase request, HttpSessionStateBase session)
        {
            string currentUrl = request["josso_current_url"];
            if (string.IsNullOrEmpty(currentUrl))
                throw new NullCurrentUrlException();

            currentUrl = currentUrl.Trim();

            return requestLoginForUrl(currentUrl, request, session);
        }

        /// <summary>
        /// the action made on the logout page
        /// </summary>
        /// <param name="request"></param>
        /// <param name="response"></param>
        public void JossoLogoutPageAction(HttpRequestBase request, HttpResponseBase response)
        {
            string currentUrl = request["josso_current_url"];
            if (string.IsNullOrEmpty(currentUrl))
                throw new NullCurrentUrlException();

            requestLogoutForUrl(currentUrl, response, request);
        }

        /// <summary>
        /// the action made by the check verification page
        /// </summary>
        /// <param name="request"></param>
        /// <param name="session"></param>
        /// <param name="response"></param>
        public void JossoCheckPageAction(HttpRequestBase request, HttpSessionStateBase session,
                                         HttpResponseBase response)
        {
            // try to get backUrl from request. If not, get it in the session.
            string backToUrl = request[JOSSO_PARTNER_APP_CTX] != null ?
                                                                          HttpUtility.UrlDecode(request[JOSSO_PARTNER_APP_CTX]) :
                                                                                                                                    (string)session["josso.backToUrl"];
            // si l'url est null on se remet sur la page racine
            if (string.IsNullOrEmpty(backToUrl))
                backToUrl = request.ApplicationPath;

            string assertionId = request["josso_assertion_id"];

            if (!string.IsNullOrEmpty(assertionId))
            {
                string ssoSession;

                if (!resolveAuthenticationAssertion(assertionId, out ssoSession)
                    || string.IsNullOrEmpty(ssoSession))
                    response.Redirect(requestLoginForUrl(backToUrl, request, session), true);
                else
                {
                    HttpCookie jossoSessionId = new HttpCookie(COOKIE_NAME);
                    jossoSessionId.Value = ssoSession;
                    response.Cookies.Add(jossoSessionId);
                    response.Redirect(backToUrl);
                }
            }
            else
                response.Redirect(requestLoginForUrl(backToUrl, request, session), true);
        }

        /// <summary>
        /// Use this function when ever you want to start user authentication.
        /// </summary>
        /// <param name="request"></param>
        /// <param name="session"></param>
        /// <param name="jossoLogin"></param>
        /// <param name="redirectUrl"></param>
        /// <returns>Returns true if the user is logged and there is no need to reset his password.</returns>
        public bool TryGetJossoLogin(
            HttpRequestBase request, HttpSessionStateBase session, out string jossoLogin, out string redirectUrl)
        {
            ZjsUser jossoUser = JossoCurrentUser(request);
            string currentUrl = request.Url.AbsoluteUri;

            jossoLogin = null;
            redirectUrl = null;

            // Access JOSSO Session to keep it alive ...
            // Check if a user is present otherwise request for authentication
            if (accessJossoSession(request) && jossoUser != null)
            {
                jossoLogin = jossoUser.Name; // ok, a user is present
                return true;
            }

            // clear the cache to not keep outdated data (because session cannot be accessed)
            if (jossoUser != null)
            {
                ZjsUser.Clear(jossoUser.SessionId);
            }

            redirectUrl = requestLoginForUrl(currentUrl, request, session);
            return false;
        }

        /// <summary>
        /// Creates a login url for the current page, use to create links to JOSSO login page
        /// </summary>
        /// <param name="request"></param>
        /// <returns>the url</returns>
        public string JossoCreateLoginUrl(HttpRequestBase request)
        {
            return string.Format("{0}{1}?josso_current_url={2}",
                                 getPageBaseUrl(request),
                                 jossoLoginPage,
                                 request.Url.AbsoluteUri);
        }

        /// <summary>
        /// Creates a logout url for the current page, use to create links to JOSSO logout page
        /// </summary>
        /// <param name="request"></param>
        /// <returns>the url</returns>
        public string JossoCreateLogoutUrl(HttpRequestBase request)
        {
            return JossoCreateLogoutUrl(request, request.Url.AbsoluteUri);
        }

        /// <summary>
        /// Creates a logout url for the current page, use to create links 
        /// to JOSSO logout page.
        /// </summary>
        /// <param name="request">The request.</param>
        /// <param name="returnURL">The desired return URL.</param>
        /// <returns>The logout URL.</returns>
        public string JossoCreateLogoutUrl(HttpRequestBase request, string returnURL)
        {
            return string.Format("{0}{1}?josso_current_url={2}",
                                 getPageBaseUrl(request),
                                 jossoLogoutPage,
                                 returnURL);
        }

        /// <summary>
        /// Tells Josso to forget cached information about the current user.
        /// </summary>
        /// <param name="request">The user request.</param>
        public void JossoInvalidateCurrentUser(HttpRequestBase request)
        {
            ZjsUser user = JossoCurrentUser(request);
            if (user != null)
                ZjsUser.Clear(user.SessionId);
        }

        /// <summary>
        /// This function returns the JOSSO Token, used to acess josso methods
        /// </summary>
        /// <param name="request"></param>
        /// <returns>the cookie</returns>
        private static HttpCookie getJossoToken(HttpRequestBase request)
        {
            // Return JOSSO Token;
            return request.Cookies[COOKIE_NAME];
        }

        /// <summary>
        /// get the url of the current page
        /// </summary>
        /// <param name="request">the current http requet object</param>
        /// <returns>the url of the page</returns>
        private static string getPageBaseUrl(HttpRequestBase request)
        {
            string url;
            string host = request.ServerVariables["server_name"];
            int port = Convert.ToInt32(request.ServerVariables["server_port"]);

            // Get protocol, server and port information
            if (request.ServerVariables["https"] == "on")
            {
                url = string.Format("https://{0}", host);
                if (port != 443)
                {
                    url = string.Format("{0}:{1}", url, port);
                }
            }
            else
            {
                url = "http://" + host;
                if (port != 80)
                {
                    url = string.Format("{0}:{1}", url, port);
                }
            }

            url = url + request.ApplicationPath;
            switch (request.ApplicationPath)
            {
                case "/":
                    return url;
                default:
                    return url + "/";
            }
        }

        /// <summary>
        /// for the checking of the security, resolve assertion
        /// </summary>
        /// <param name="id">the id returns from the JOSSO Gateway</param>
        /// <param name="ssoSession">the output is the sso session of the user</param>
        /// <returns>true if ok</returns>
        private bool resolveAuthenticationAssertion(string id, out string ssoSession)
        {
            return Josso.ResolveAuthenticationAssertion(id, out ssoSession);
        }

        /// <summary>
        /// This method accesss the session associated to the received id.
        /// This resets the session last access time and updates the access count.
        /// </summary>
        /// <param name="request"></param>
        private bool accessJossoSession(HttpRequestBase request)
        {
            HttpCookie jossoToken = getJossoToken(request);
            if (jossoToken == null)
                return false;
            return Josso.AccessSession(jossoToken.Value);
        }

        /// <summary>
        /// Use this fonction when you want a log out with a custom current url
        /// </summary>
        /// <param name="currentUrl">the current url to log out</param>
        /// <param name="response">the http response for redirection</param>
        /// <param name="request">the current request for cookie deletion</param>
        private void requestLogoutForUrl(string currentUrl, HttpResponseBase response, HttpRequestBase request)
        {
            if (response.IsRequestBeingRedirected)
                return;

            HttpCookie session = getJossoToken(request);
            if (session != null)
            {
                session.Expires = DateTime.Now.AddDays(-1);
                response.Cookies.Add(session);
                Josso.GlobalLogout(session.Value);
            }

            string backToUrl = currentUrl;
            string loginUrl = _gatewayLogout + "?josso_back_to=" + backToUrl;

            response.Redirect(loginUrl);
        }

        /// <summary>
        /// Use this function when ever you want to start user authentication with a custom
        /// current url
        /// </summary>
        /// <param name="currentUrl">the url that will be back after authentication</param>
        /// <param name="request"></param>
        /// <param name="session"></param>
        private string requestLoginForUrl(string currentUrl, HttpRequestBase request, HttpSessionStateBase session)
        {
            string backToUrl = getPageBaseUrl(request);

            // Store original URL
            session["josso.backToUrl"] = currentUrl;

            backToUrl = backToUrl + jossoCheckPage;

            // build the login url
            string loginUrl = _gatewayLogin + "?josso_back_to=" + backToUrl;
            string host = request.ServerVariables["server_name"];
            loginUrl = loginUrl + "&josso_partnerapp_host=" + host + "&" + JOSSO_PARTNER_APP_CTX + "=" + HttpUtility.UrlEncode(currentUrl);

            return loginUrl;
        }
    }
}