﻿using System.Collections.Generic;
using System.Net;
using System.Web.Services.Protocols;
using Josso.Agent.Lib.IdentityManager;
using Josso.Agent.Lib.IdentityProvider;
using Josso.Agent.Lib.SessionManager;

namespace Josso.Agent.Lib
{
    /// <summary>
    /// this class interface the josso gateway to the agent
    /// it mainly requests the webservices
    /// </summary>
    internal class ZjsJosso
    {
        /// <summary>
        /// the web service for the identity provider
        /// </summary>
        private readonly SSOIdentityProviderWS _identityProvider;

        /// <summary>
        /// the web service for the identity manager
        /// </summary>
        private readonly SSOIdentityManagerWS _identityManager;

        /// <summary>
        /// the web service for the session manager
        /// </summary>
        private readonly SSOSessionManagerWS _sessionManager;

        /// <summary>
        /// instantiate the web service
        /// </summary>
        /// <param name="gatewayURL">the url of the gateway</param>
        public ZjsJosso(string gatewayURL)
        {
            _identityProvider = new SSOIdentityProviderWS();
            _identityProvider.Url = string.Format("{0}/josso/services/SSOIdentityProvider", gatewayURL);
            _identityManager = new SSOIdentityManagerWS();
            _identityManager.Url = string.Format("{0}/josso/services/SSOIdentityManager", gatewayURL);
            _sessionManager = new SSOSessionManagerWS();
            _sessionManager.Url = string.Format("{0}/josso/services/SSOSessionManager", gatewayURL);
        }

        /// <summary>
        /// Disable the validation of the remote certificate
        /// </summary>
        internal bool DisableCertificate
        {
            set
            {
                if (value)
                {
                    ServicePointManager.ServerCertificateValidationCallback =
                        (sender, certificate, chain, sslPolicyErrors) => true;
                }
                else
                {
                    ServicePointManager.ServerCertificateValidationCallback = null;
                }
            }
        }

        /// <summary>
        /// resolve the authentication assertion
        /// </summary>
        /// <param name="assertionId">the assertion id gets from the josso gateway</param>
        /// <param name="ssoSession">the sso session of the user</param>
        /// <returns>true if ok</returns>
        public virtual bool ResolveAuthenticationAssertion(string assertionId, out string ssoSession)
        {
            ResolveAuthenticationAssertionRequestType request = new ResolveAuthenticationAssertionRequestType();
            request.assertionId = assertionId;
            try
            {
                ssoSession = _identityProvider.resolveAuthenticationAssertion(request).ssoSessionId;
                return true;
            }
            catch (SoapException)
            {
                ssoSession = null;
                return false;
            }
        }

        public virtual Dictionary<string, string> GetProperties(string ssoSession)
        {
            FindUserInSessionRequestType request = new FindUserInSessionRequestType();
            request.ssoSessionId = ssoSession;
            Dictionary<string, string> properties = new Dictionary<string, string>();
            foreach (var property in _identityManager.findUserInSession(request).SSOUser.properties)
            {
                properties[property.name] = property.value;
            }

            return properties;
        }

        /// <summary>
        /// identify the user
        /// </summary>
        /// <param name="user">the user</param>
        /// <param name="password">the password</param>
        /// <param name="ssoSession">the sso session</param>
        /// <returns>true if identify</returns>
        public virtual bool IdentifyUser(string user, string password, out string ssoSession)
        {
            AssertIdentityWithSimpleAuthenticationRequestType request2 =
                new AssertIdentityWithSimpleAuthenticationRequestType
                    {
                        username = user,
                        password = password,
                        securityDomain = "josso"
                    };

            AssertIdentityWithSimpleAuthenticationResponseType result;
            ssoSession = null;
            try
            {
                result = _identityProvider.assertIdentityWithSimpleAuthentication(request2);
            }
            catch (SoapException)
            {
                return false;
            }

            return ResolveAuthenticationAssertion(result.assertionId, out ssoSession);
        }

        /// <summary>
        /// get the user name according to the session id
        /// </summary>
        /// <param name="sessionId"></param>
        /// <returns>the user name</returns>
        public virtual string GetUserName(string sessionId)
        {
            FindUserInSessionRequestType request = new FindUserInSessionRequestType
                                                       {
                                                           ssoSessionId = sessionId
                                                       };

            return _identityManager.findUserInSession(request).SSOUser.name;
        }

        /// <summary>
        /// access the session to keep it alive
        /// </summary>
        /// <param name="sessionId"></param>
        /// <returns>true if the session exists</returns>
        public virtual bool AccessSession(string sessionId)
        {
            AccessSessionRequestType request = new AccessSessionRequestType
                                                   {
                                                       ssoSessionId = sessionId
                                                   };

            try
            {
                _sessionManager.accessSession(request);
                return true;
            }
            catch (SoapException)
            {
                return false; // mean there is no session
            }
        }

        /// <summary>
        /// Global signout of the session
        /// </summary>
        /// <param name="sessionId"></param>
        public virtual void GlobalLogout(string sessionId)
        {
            GlobalSignoffRequestType request = new GlobalSignoffRequestType
                                                   {
                                                       ssoSessionId = sessionId
                                                   };

            try
            {
                _identityProvider.globalSignoff(request);
            }
            catch (SoapException)
            {
                return; // mean there is no session
            }
        }
    }
}